/*
  VisualHash - A Visual Hash Calculator
  Copyright (C) 2005 Dominik Reichl <oss@dominik-reichl.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using System.Text;
using System.IO;
using System.Diagnostics;

using VisualHash.ReHashLib;

namespace VisualHash
{
	public class MainForm : System.Windows.Forms.Form
	{
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.Panel panelHashFilesTopInner;
		private System.Windows.Forms.RichTextBox richTextToHash;
		private System.Windows.Forms.TabPage tabPageHashText;
		private System.Windows.Forms.RichTextBox richTextResults;
		private System.Windows.Forms.FolderBrowserDialog folderBrowserToAdd;
		private System.Windows.Forms.MenuItem menuInfo;
		private System.Windows.Forms.ContextMenu ctxFileList;
		private System.Windows.Forms.RichTextBox richFileResults;
		private System.Windows.Forms.TabControl tabPages;
		private System.Windows.Forms.MenuItem menuFileCtxClear;
		private System.Windows.Forms.LinkLabel linkAlgoNone;
		private System.Windows.Forms.MenuItem menuFileCtxAddDir;
		private System.Windows.Forms.StatusBar statusMain;
		private System.Windows.Forms.Panel panelHashTextTopIntro;
		private System.Windows.Forms.MenuItem menuFileCtxRemove;
		private System.Windows.Forms.Button btnHashText;
		private System.Windows.Forms.Panel panelFilesProgressSpaceTop;
		private System.Windows.Forms.TabPage tabPageOptions;
		private System.Windows.Forms.LinkLabel linkAlgoAll;
		private System.Windows.Forms.MenuItem menuTextCtxCopy;
		private System.Windows.Forms.Panel panelHashFilesTopControls;
		private System.Windows.Forms.MenuItem menuFileExit;
		private System.Windows.Forms.OpenFileDialog openFileToAdd;
		private System.Windows.Forms.Panel panelAlgoTop;
		private System.Windows.Forms.MenuItem menuTextCtxSep1;
		private System.Windows.Forms.Panel panelHashFilesLeftButton;
		private System.Windows.Forms.MenuItem menuTextCtxPaste;
		private System.Windows.Forms.ListView lvFilesToHash;
		private System.Windows.Forms.Label lblBaseIndentText;
		private System.Windows.Forms.Button btnHashFiles;
		private System.Windows.Forms.MainMenu menuMain;
		private System.Windows.Forms.ListView lvAlgorithms;
		private System.Windows.Forms.MenuItem menuFileCtxAddFile;
		private System.Windows.Forms.Panel panelFilesProgressSpaceBottom;
		private System.Windows.Forms.TabPage tabPageHashFiles;
		private System.Windows.Forms.CheckBox checkUpperCaseHex;
		private System.Windows.Forms.MenuItem menuTextCtxClear;
		private System.Windows.Forms.Panel panelHashTextTop;
		private System.Windows.Forms.MenuItem menuFile;
		private System.Windows.Forms.Panel panelAlgoBottom;
		private System.Windows.Forms.Label stcHashFilesIntro;
		private System.Windows.Forms.ContextMenu ctxTextView;
		private System.Windows.Forms.GroupBox groupReportOptions;
		private System.Windows.Forms.TabPage tabPageAlgorithms;
		private System.Windows.Forms.MenuItem menuFileCtxSep2;
		private System.Windows.Forms.ImageList imageListMain;
		private System.Windows.Forms.Label stcHashTextIntro;
		private System.Windows.Forms.MenuItem menuTextCtxCopyAll;
		private System.Windows.Forms.ProgressBar progressFiles;
		private System.Windows.Forms.Panel panelHashFilesTop;
		private System.Windows.Forms.Label stcAlgoIntro;
		private System.Windows.Forms.Label lblBaseIndentPostInfo;
		private System.Windows.Forms.MenuItem menuFileCtxSep1;
		private System.Windows.Forms.NumericUpDown numBaseIndent;
		private System.Windows.Forms.Panel panelHashTextTopControls;
		private System.Windows.Forms.MenuItem menuInfoAbout;
		
		private HashManager m_hm = new HashManager();
		private string m_strCfgFile = string.Empty;
		private PrivateXmlConfig m_xmlConfig = new PrivateXmlConfig();

		public MainForm()
		{
			InitializeComponent();

			m_strCfgFile = System.Reflection.Assembly.GetExecutingAssembly().Location;
			if(m_strCfgFile.EndsWith(".exe"))
				m_strCfgFile = m_strCfgFile.Substring(0, m_strCfgFile.Length - 4);
			m_strCfgFile += ".config.xml";

			m_xmlConfig = new PrivateXmlConfig();
			m_xmlConfig.LoadFromFile(m_strCfgFile);

			m_hm.InitModules();
	
			lvFilesToHash.Columns.Add("Files To Hash", 100, HorizontalAlignment.Left);

			lvAlgorithms.Columns.Add("Algorithm", 200, HorizontalAlignment.Left);
			lvAlgorithms.Columns.Add("Hash Size", 100, HorizontalAlignment.Left);
			lvAlgorithms.Columns.Add("Specification / Reference", 400, HorizontalAlignment.Left);

			ReHashModule h;
			string strKeyName, strKeyValue;
			for(int i = 0; i < m_hm.m_hashes.Count; i++)
			{
				h = (ReHashModule)m_hm.m_hashes[i];
				
				strKeyName = "Enable-" + h.strShortName;
				strKeyValue = m_xmlConfig.GetValue(strKeyName);
				if(strKeyValue == "False") h.bEnabled = false;
				else h.bEnabled = true;

				ListViewItem lvi = lvAlgorithms.Items.Add(h.strName);
				lvi.SubItems.Add(h.uHashSize.ToString() + " bit");
				lvi.SubItems.Add(h.strRef);
				lvi.Checked = h.bEnabled;

				uint u = h.Test();
				if(u != 0)
					MessageBox.Show("Self-test of algorithm \"" +
						h.strName +
						"\" failed!\r\n\r\nThe hash digests of this algorithm can therefore be incorrect.\r\n\r\nError code: " +
						u,
						"Self-test failed", MessageBoxButtons.OK, MessageBoxIcon.Warning);
			}
			
			strKeyValue = m_xmlConfig.GetValue("Results.UpperCaseHex");
			if(strKeyValue == "False") checkUpperCaseHex.Checked = false;
			else checkUpperCaseHex.Checked = true;

			strKeyValue = m_xmlConfig.GetValue("Results.BaseIndent");
			try { numBaseIndent.Value = decimal.Parse(strKeyValue); }
			catch(Exception) { numBaseIndent.Value = 8; }

			SetMainStatusText("");

			ProcessResize();
		}

		[STAThread]
		public static void Main(string[] args)
		{
			Application.EnableVisualStyles();
			Application.DoEvents();
			Application.Run(new MainForm());
		}
		
		#region Windows Forms Designer generated code
		/// <summary>
		/// This method is required for Windows Forms designer support.
		/// Do not change the method contents inside the source code editor. The Forms designer might
		/// not be able to load this method if it was changed manually.
		/// </summary>
		private void InitializeComponent() {
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MainForm));
			this.menuInfoAbout = new System.Windows.Forms.MenuItem();
			this.panelHashTextTopControls = new System.Windows.Forms.Panel();
			this.numBaseIndent = new System.Windows.Forms.NumericUpDown();
			this.menuFileCtxSep1 = new System.Windows.Forms.MenuItem();
			this.lblBaseIndentPostInfo = new System.Windows.Forms.Label();
			this.stcAlgoIntro = new System.Windows.Forms.Label();
			this.panelHashFilesTop = new System.Windows.Forms.Panel();
			this.progressFiles = new System.Windows.Forms.ProgressBar();
			this.menuTextCtxCopyAll = new System.Windows.Forms.MenuItem();
			this.stcHashTextIntro = new System.Windows.Forms.Label();
			this.imageListMain = new System.Windows.Forms.ImageList(this.components);
			this.menuFileCtxSep2 = new System.Windows.Forms.MenuItem();
			this.tabPageAlgorithms = new System.Windows.Forms.TabPage();
			this.groupReportOptions = new System.Windows.Forms.GroupBox();
			this.ctxTextView = new System.Windows.Forms.ContextMenu();
			this.stcHashFilesIntro = new System.Windows.Forms.Label();
			this.panelAlgoBottom = new System.Windows.Forms.Panel();
			this.menuFile = new System.Windows.Forms.MenuItem();
			this.panelHashTextTop = new System.Windows.Forms.Panel();
			this.menuTextCtxClear = new System.Windows.Forms.MenuItem();
			this.checkUpperCaseHex = new System.Windows.Forms.CheckBox();
			this.tabPageHashFiles = new System.Windows.Forms.TabPage();
			this.panelFilesProgressSpaceBottom = new System.Windows.Forms.Panel();
			this.menuFileCtxAddFile = new System.Windows.Forms.MenuItem();
			this.lvAlgorithms = new System.Windows.Forms.ListView();
			this.menuMain = new System.Windows.Forms.MainMenu();
			this.btnHashFiles = new System.Windows.Forms.Button();
			this.lblBaseIndentText = new System.Windows.Forms.Label();
			this.lvFilesToHash = new System.Windows.Forms.ListView();
			this.menuTextCtxPaste = new System.Windows.Forms.MenuItem();
			this.panelHashFilesLeftButton = new System.Windows.Forms.Panel();
			this.menuTextCtxSep1 = new System.Windows.Forms.MenuItem();
			this.panelAlgoTop = new System.Windows.Forms.Panel();
			this.openFileToAdd = new System.Windows.Forms.OpenFileDialog();
			this.menuFileExit = new System.Windows.Forms.MenuItem();
			this.panelHashFilesTopControls = new System.Windows.Forms.Panel();
			this.menuTextCtxCopy = new System.Windows.Forms.MenuItem();
			this.linkAlgoAll = new System.Windows.Forms.LinkLabel();
			this.tabPageOptions = new System.Windows.Forms.TabPage();
			this.panelFilesProgressSpaceTop = new System.Windows.Forms.Panel();
			this.btnHashText = new System.Windows.Forms.Button();
			this.menuFileCtxRemove = new System.Windows.Forms.MenuItem();
			this.panelHashTextTopIntro = new System.Windows.Forms.Panel();
			this.statusMain = new System.Windows.Forms.StatusBar();
			this.menuFileCtxAddDir = new System.Windows.Forms.MenuItem();
			this.linkAlgoNone = new System.Windows.Forms.LinkLabel();
			this.menuFileCtxClear = new System.Windows.Forms.MenuItem();
			this.tabPages = new System.Windows.Forms.TabControl();
			this.richFileResults = new System.Windows.Forms.RichTextBox();
			this.ctxFileList = new System.Windows.Forms.ContextMenu();
			this.menuInfo = new System.Windows.Forms.MenuItem();
			this.folderBrowserToAdd = new System.Windows.Forms.FolderBrowserDialog();
			this.richTextResults = new System.Windows.Forms.RichTextBox();
			this.tabPageHashText = new System.Windows.Forms.TabPage();
			this.richTextToHash = new System.Windows.Forms.RichTextBox();
			this.panelHashFilesTopInner = new System.Windows.Forms.Panel();
			this.panelHashTextTopControls.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.numBaseIndent)).BeginInit();
			this.panelHashFilesTop.SuspendLayout();
			this.tabPageAlgorithms.SuspendLayout();
			this.groupReportOptions.SuspendLayout();
			this.panelAlgoBottom.SuspendLayout();
			this.panelHashTextTop.SuspendLayout();
			this.tabPageHashFiles.SuspendLayout();
			this.panelHashFilesLeftButton.SuspendLayout();
			this.panelAlgoTop.SuspendLayout();
			this.panelHashFilesTopControls.SuspendLayout();
			this.tabPageOptions.SuspendLayout();
			this.panelHashTextTopIntro.SuspendLayout();
			this.tabPages.SuspendLayout();
			this.tabPageHashText.SuspendLayout();
			this.panelHashFilesTopInner.SuspendLayout();
			this.SuspendLayout();
			// 
			// menuInfoAbout
			// 
			this.menuInfoAbout.Index = 0;
			this.menuInfoAbout.Text = "&About VisualHash...";
			this.menuInfoAbout.Click += new System.EventHandler(this.OnMenuInfoAbout);
			// 
			// panelHashTextTopControls
			// 
			this.panelHashTextTopControls.Controls.Add(this.btnHashText);
			this.panelHashTextTopControls.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.panelHashTextTopControls.Location = new System.Drawing.Point(0, 176);
			this.panelHashTextTopControls.Name = "panelHashTextTopControls";
			this.panelHashTextTopControls.Size = new System.Drawing.Size(544, 32);
			this.panelHashTextTopControls.TabIndex = 1;
			// 
			// numBaseIndent
			// 
			this.numBaseIndent.Location = new System.Drawing.Point(80, 24);
			this.numBaseIndent.Name = "numBaseIndent";
			this.numBaseIndent.Size = new System.Drawing.Size(48, 20);
			this.numBaseIndent.TabIndex = 2;
			this.numBaseIndent.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
			this.numBaseIndent.Value = new System.Decimal(new int[] {
						8,
						0,
						0,
						0});
			// 
			// menuFileCtxSep1
			// 
			this.menuFileCtxSep1.Index = 2;
			this.menuFileCtxSep1.Text = "-";
			// 
			// lblBaseIndentPostInfo
			// 
			this.lblBaseIndentPostInfo.Location = new System.Drawing.Point(131, 27);
			this.lblBaseIndentPostInfo.Name = "lblBaseIndentPostInfo";
			this.lblBaseIndentPostInfo.Size = new System.Drawing.Size(64, 16);
			this.lblBaseIndentPostInfo.TabIndex = 3;
			this.lblBaseIndentPostInfo.Text = "characters";
			// 
			// stcAlgoIntro
			// 
			this.stcAlgoIntro.Location = new System.Drawing.Point(2, 8);
			this.stcAlgoIntro.Name = "stcAlgoIntro";
			this.stcAlgoIntro.Size = new System.Drawing.Size(360, 18);
			this.stcAlgoIntro.TabIndex = 0;
			this.stcAlgoIntro.Text = "Here you can select the algorithms you wish to use:";
			// 
			// panelHashFilesTop
			// 
			this.panelHashFilesTop.Controls.Add(this.lvFilesToHash);
			this.panelHashFilesTop.Controls.Add(this.panelHashFilesTopControls);
			this.panelHashFilesTop.Controls.Add(this.panelHashFilesTopInner);
			this.panelHashFilesTop.Dock = System.Windows.Forms.DockStyle.Top;
			this.panelHashFilesTop.Location = new System.Drawing.Point(0, 0);
			this.panelHashFilesTop.Name = "panelHashFilesTop";
			this.panelHashFilesTop.Size = new System.Drawing.Size(544, 208);
			this.panelHashFilesTop.TabIndex = 0;
			// 
			// progressFiles
			// 
			this.progressFiles.Dock = System.Windows.Forms.DockStyle.Fill;
			this.progressFiles.Location = new System.Drawing.Point(120, 8);
			this.progressFiles.Name = "progressFiles";
			this.progressFiles.Size = new System.Drawing.Size(424, 18);
			this.progressFiles.TabIndex = 4;
			// 
			// menuTextCtxCopyAll
			// 
			this.menuTextCtxCopyAll.Index = 3;
			this.menuTextCtxCopyAll.Text = "Copy All To Clipboard";
			this.menuTextCtxCopyAll.Click += new System.EventHandler(this.OnMenuTextCtxCopyAll);
			// 
			// stcHashTextIntro
			// 
			this.stcHashTextIntro.Location = new System.Drawing.Point(2, 8);
			this.stcHashTextIntro.Name = "stcHashTextIntro";
			this.stcHashTextIntro.Size = new System.Drawing.Size(208, 18);
			this.stcHashTextIntro.TabIndex = 0;
			this.stcHashTextIntro.Text = "Enter the text you want to hash here:";
			// 
			// imageListMain
			// 
			this.imageListMain.ColorDepth = System.Windows.Forms.ColorDepth.Depth24Bit;
			this.imageListMain.ImageSize = new System.Drawing.Size(16, 16);
			this.imageListMain.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageListMain.ImageStream")));
			this.imageListMain.TransparentColor = System.Drawing.Color.Magenta;
			// 
			// menuFileCtxSep2
			// 
			this.menuFileCtxSep2.Index = 4;
			this.menuFileCtxSep2.Text = "-";
			// 
			// tabPageAlgorithms
			// 
			this.tabPageAlgorithms.Controls.Add(this.lvAlgorithms);
			this.tabPageAlgorithms.Controls.Add(this.panelAlgoTop);
			this.tabPageAlgorithms.Controls.Add(this.panelAlgoBottom);
			this.tabPageAlgorithms.ImageIndex = 2;
			this.tabPageAlgorithms.Location = new System.Drawing.Point(4, 23);
			this.tabPageAlgorithms.Name = "tabPageAlgorithms";
			this.tabPageAlgorithms.Size = new System.Drawing.Size(544, 383);
			this.tabPageAlgorithms.TabIndex = 2;
			this.tabPageAlgorithms.Text = "Algorithms";
			this.tabPageAlgorithms.ToolTipText = "Select the algorithms to use.";
			// 
			// groupReportOptions
			// 
			this.groupReportOptions.Controls.Add(this.checkUpperCaseHex);
			this.groupReportOptions.Controls.Add(this.lblBaseIndentPostInfo);
			this.groupReportOptions.Controls.Add(this.lblBaseIndentText);
			this.groupReportOptions.Controls.Add(this.numBaseIndent);
			this.groupReportOptions.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.groupReportOptions.Location = new System.Drawing.Point(8, 8);
			this.groupReportOptions.Name = "groupReportOptions";
			this.groupReportOptions.Size = new System.Drawing.Size(208, 88);
			this.groupReportOptions.TabIndex = 1;
			this.groupReportOptions.TabStop = false;
			this.groupReportOptions.Text = "Report Options";
			// 
			// ctxTextView
			// 
			this.ctxTextView.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
						this.menuTextCtxCopy,
						this.menuTextCtxPaste,
						this.menuTextCtxSep1,
						this.menuTextCtxCopyAll,
						this.menuTextCtxClear});
			// 
			// stcHashFilesIntro
			// 
			this.stcHashFilesIntro.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.stcHashFilesIntro.Location = new System.Drawing.Point(2, 8);
			this.stcHashFilesIntro.Name = "stcHashFilesIntro";
			this.stcHashFilesIntro.Size = new System.Drawing.Size(272, 14);
			this.stcHashFilesIntro.TabIndex = 1;
			this.stcHashFilesIntro.Text = "Drag&&drop the files you want to hash into this list:";
			// 
			// panelAlgoBottom
			// 
			this.panelAlgoBottom.Controls.Add(this.linkAlgoNone);
			this.panelAlgoBottom.Controls.Add(this.linkAlgoAll);
			this.panelAlgoBottom.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.panelAlgoBottom.Location = new System.Drawing.Point(0, 335);
			this.panelAlgoBottom.Name = "panelAlgoBottom";
			this.panelAlgoBottom.Size = new System.Drawing.Size(544, 48);
			this.panelAlgoBottom.TabIndex = 0;
			// 
			// menuFile
			// 
			this.menuFile.Index = 0;
			this.menuFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
						this.menuFileExit});
			this.menuFile.Text = "&File";
			// 
			// panelHashTextTop
			// 
			this.panelHashTextTop.Controls.Add(this.richTextToHash);
			this.panelHashTextTop.Controls.Add(this.panelHashTextTopControls);
			this.panelHashTextTop.Controls.Add(this.panelHashTextTopIntro);
			this.panelHashTextTop.Dock = System.Windows.Forms.DockStyle.Top;
			this.panelHashTextTop.Location = new System.Drawing.Point(0, 0);
			this.panelHashTextTop.Name = "panelHashTextTop";
			this.panelHashTextTop.Size = new System.Drawing.Size(544, 208);
			this.panelHashTextTop.TabIndex = 0;
			// 
			// menuTextCtxClear
			// 
			this.menuTextCtxClear.Index = 4;
			this.menuTextCtxClear.Text = "Clear";
			this.menuTextCtxClear.Click += new System.EventHandler(this.OnMenuTextCtxClear);
			// 
			// checkUpperCaseHex
			// 
			this.checkUpperCaseHex.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.checkUpperCaseHex.Location = new System.Drawing.Point(8, 64);
			this.checkUpperCaseHex.Name = "checkUpperCaseHex";
			this.checkUpperCaseHex.Size = new System.Drawing.Size(184, 16);
			this.checkUpperCaseHex.TabIndex = 4;
			this.checkUpperCaseHex.Text = "Upper case hex characters";
			// 
			// tabPageHashFiles
			// 
			this.tabPageHashFiles.Controls.Add(this.richFileResults);
			this.tabPageHashFiles.Controls.Add(this.panelHashFilesTop);
			this.tabPageHashFiles.ImageIndex = 0;
			this.tabPageHashFiles.Location = new System.Drawing.Point(4, 23);
			this.tabPageHashFiles.Name = "tabPageHashFiles";
			this.tabPageHashFiles.Size = new System.Drawing.Size(544, 383);
			this.tabPageHashFiles.TabIndex = 1;
			this.tabPageHashFiles.Text = "Hash Files";
			this.tabPageHashFiles.ToolTipText = "Compute hashsums of files.";
			// 
			// panelFilesProgressSpaceBottom
			// 
			this.panelFilesProgressSpaceBottom.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.panelFilesProgressSpaceBottom.Location = new System.Drawing.Point(120, 26);
			this.panelFilesProgressSpaceBottom.Name = "panelFilesProgressSpaceBottom";
			this.panelFilesProgressSpaceBottom.Size = new System.Drawing.Size(424, 8);
			this.panelFilesProgressSpaceBottom.TabIndex = 6;
			// 
			// menuFileCtxAddFile
			// 
			this.menuFileCtxAddFile.Index = 0;
			this.menuFileCtxAddFile.Text = "Add &File...";
			this.menuFileCtxAddFile.Click += new System.EventHandler(this.OnMenuFileCtxAddFile);
			// 
			// lvAlgorithms
			// 
			this.lvAlgorithms.CheckBoxes = true;
			this.lvAlgorithms.Dock = System.Windows.Forms.DockStyle.Fill;
			this.lvAlgorithms.FullRowSelect = true;
			this.lvAlgorithms.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.lvAlgorithms.Location = new System.Drawing.Point(0, 28);
			this.lvAlgorithms.Name = "lvAlgorithms";
			this.lvAlgorithms.Size = new System.Drawing.Size(544, 307);
			this.lvAlgorithms.TabIndex = 2;
			this.lvAlgorithms.View = System.Windows.Forms.View.Details;
			this.lvAlgorithms.ItemCheck += new System.Windows.Forms.ItemCheckEventHandler(this.OnItemCheckAlgorithms);
			// 
			// menuMain
			// 
			this.menuMain.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
						this.menuFile,
						this.menuInfo});
			// 
			// btnHashFiles
			// 
			this.btnHashFiles.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnHashFiles.Location = new System.Drawing.Point(0, 6);
			this.btnHashFiles.Name = "btnHashFiles";
			this.btnHashFiles.Size = new System.Drawing.Size(116, 24);
			this.btnHashFiles.TabIndex = 2;
			this.btnHashFiles.Text = "Compute Hashes";
			this.btnHashFiles.Click += new System.EventHandler(this.OnClickHashFiles);
			// 
			// lblBaseIndentText
			// 
			this.lblBaseIndentText.Location = new System.Drawing.Point(8, 26);
			this.lblBaseIndentText.Name = "lblBaseIndentText";
			this.lblBaseIndentText.Size = new System.Drawing.Size(72, 16);
			this.lblBaseIndentText.TabIndex = 0;
			this.lblBaseIndentText.Text = "Base Indent:";
			// 
			// lvFilesToHash
			// 
			this.lvFilesToHash.AllowDrop = true;
			this.lvFilesToHash.ContextMenu = this.ctxFileList;
			this.lvFilesToHash.Dock = System.Windows.Forms.DockStyle.Fill;
			this.lvFilesToHash.GridLines = true;
			this.lvFilesToHash.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.lvFilesToHash.Location = new System.Drawing.Point(0, 28);
			this.lvFilesToHash.Name = "lvFilesToHash";
			this.lvFilesToHash.Size = new System.Drawing.Size(544, 146);
			this.lvFilesToHash.TabIndex = 0;
			this.lvFilesToHash.View = System.Windows.Forms.View.Details;
			this.lvFilesToHash.DragDrop += new System.Windows.Forms.DragEventHandler(this.OnFilesLvDrop);
			this.lvFilesToHash.DragEnter += new System.Windows.Forms.DragEventHandler(this.OnFilesLvDragEnter);
			// 
			// menuTextCtxPaste
			// 
			this.menuTextCtxPaste.Index = 1;
			this.menuTextCtxPaste.Text = "&Paste";
			this.menuTextCtxPaste.Click += new System.EventHandler(this.OnMenuTextCtxPaste);
			// 
			// panelHashFilesLeftButton
			// 
			this.panelHashFilesLeftButton.Controls.Add(this.btnHashFiles);
			this.panelHashFilesLeftButton.Dock = System.Windows.Forms.DockStyle.Left;
			this.panelHashFilesLeftButton.Location = new System.Drawing.Point(0, 0);
			this.panelHashFilesLeftButton.Name = "panelHashFilesLeftButton";
			this.panelHashFilesLeftButton.Size = new System.Drawing.Size(120, 34);
			this.panelHashFilesLeftButton.TabIndex = 3;
			// 
			// menuTextCtxSep1
			// 
			this.menuTextCtxSep1.Index = 2;
			this.menuTextCtxSep1.Text = "-";
			// 
			// panelAlgoTop
			// 
			this.panelAlgoTop.Controls.Add(this.stcAlgoIntro);
			this.panelAlgoTop.Dock = System.Windows.Forms.DockStyle.Top;
			this.panelAlgoTop.Location = new System.Drawing.Point(0, 0);
			this.panelAlgoTop.Name = "panelAlgoTop";
			this.panelAlgoTop.Size = new System.Drawing.Size(544, 28);
			this.panelAlgoTop.TabIndex = 1;
			// 
			// openFileToAdd
			// 
			this.openFileToAdd.AddExtension = false;
			this.openFileToAdd.Filter = "All Files (*.*)|*.*";
			this.openFileToAdd.Title = "Select a file to hash";
			// 
			// menuFileExit
			// 
			this.menuFileExit.Index = 0;
			this.menuFileExit.Text = "&Exit";
			this.menuFileExit.Click += new System.EventHandler(this.OnMenuFileExit);
			// 
			// panelHashFilesTopControls
			// 
			this.panelHashFilesTopControls.Controls.Add(this.progressFiles);
			this.panelHashFilesTopControls.Controls.Add(this.panelFilesProgressSpaceTop);
			this.panelHashFilesTopControls.Controls.Add(this.panelFilesProgressSpaceBottom);
			this.panelHashFilesTopControls.Controls.Add(this.panelHashFilesLeftButton);
			this.panelHashFilesTopControls.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.panelHashFilesTopControls.Location = new System.Drawing.Point(0, 174);
			this.panelHashFilesTopControls.Name = "panelHashFilesTopControls";
			this.panelHashFilesTopControls.Size = new System.Drawing.Size(544, 34);
			this.panelHashFilesTopControls.TabIndex = 3;
			// 
			// menuTextCtxCopy
			// 
			this.menuTextCtxCopy.Index = 0;
			this.menuTextCtxCopy.Text = "&Copy";
			this.menuTextCtxCopy.Click += new System.EventHandler(this.OnMenuTextCtxCopy);
			// 
			// linkAlgoAll
			// 
			this.linkAlgoAll.Location = new System.Drawing.Point(2, 8);
			this.linkAlgoAll.Name = "linkAlgoAll";
			this.linkAlgoAll.Size = new System.Drawing.Size(184, 14);
			this.linkAlgoAll.TabIndex = 0;
			this.linkAlgoAll.TabStop = true;
			this.linkAlgoAll.Text = "Select All Algorithms";
			this.linkAlgoAll.LinkClicked += new System.Windows.Forms.LinkLabelLinkClickedEventHandler(this.OnClickAlgoAll);
			// 
			// tabPageOptions
			// 
			this.tabPageOptions.Controls.Add(this.groupReportOptions);
			this.tabPageOptions.ImageIndex = 3;
			this.tabPageOptions.Location = new System.Drawing.Point(4, 23);
			this.tabPageOptions.Name = "tabPageOptions";
			this.tabPageOptions.Size = new System.Drawing.Size(544, 383);
			this.tabPageOptions.TabIndex = 3;
			this.tabPageOptions.Text = "Options";
			this.tabPageOptions.ToolTipText = "General options.";
			// 
			// panelFilesProgressSpaceTop
			// 
			this.panelFilesProgressSpaceTop.Dock = System.Windows.Forms.DockStyle.Top;
			this.panelFilesProgressSpaceTop.Location = new System.Drawing.Point(120, 0);
			this.panelFilesProgressSpaceTop.Name = "panelFilesProgressSpaceTop";
			this.panelFilesProgressSpaceTop.Size = new System.Drawing.Size(424, 8);
			this.panelFilesProgressSpaceTop.TabIndex = 5;
			// 
			// btnHashText
			// 
			this.btnHashText.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnHashText.Location = new System.Drawing.Point(0, 4);
			this.btnHashText.Name = "btnHashText";
			this.btnHashText.Size = new System.Drawing.Size(116, 24);
			this.btnHashText.TabIndex = 2;
			this.btnHashText.Text = "Compute Hashes";
			this.btnHashText.Click += new System.EventHandler(this.OnClickHashText);
			// 
			// menuFileCtxRemove
			// 
			this.menuFileCtxRemove.Index = 3;
			this.menuFileCtxRemove.Text = "&Remove Selected From List";
			this.menuFileCtxRemove.Click += new System.EventHandler(this.OnMenuFileCtxRemove);
			// 
			// panelHashTextTopIntro
			// 
			this.panelHashTextTopIntro.Controls.Add(this.stcHashTextIntro);
			this.panelHashTextTopIntro.Dock = System.Windows.Forms.DockStyle.Top;
			this.panelHashTextTopIntro.Location = new System.Drawing.Point(0, 0);
			this.panelHashTextTopIntro.Name = "panelHashTextTopIntro";
			this.panelHashTextTopIntro.Size = new System.Drawing.Size(544, 28);
			this.panelHashTextTopIntro.TabIndex = 0;
			// 
			// statusMain
			// 
			this.statusMain.Location = new System.Drawing.Point(0, 410);
			this.statusMain.Name = "statusMain";
			this.statusMain.Size = new System.Drawing.Size(552, 19);
			this.statusMain.TabIndex = 0;
			this.statusMain.Text = "statusMain";
			// 
			// menuFileCtxAddDir
			// 
			this.menuFileCtxAddDir.Index = 1;
			this.menuFileCtxAddDir.Text = "Add &Directory...";
			this.menuFileCtxAddDir.Click += new System.EventHandler(this.OnMenuFileCtxAddDir);
			// 
			// linkAlgoNone
			// 
			this.linkAlgoNone.Location = new System.Drawing.Point(2, 24);
			this.linkAlgoNone.Name = "linkAlgoNone";
			this.linkAlgoNone.Size = new System.Drawing.Size(176, 14);
			this.linkAlgoNone.TabIndex = 1;
			this.linkAlgoNone.TabStop = true;
			this.linkAlgoNone.Text = "Unselect All Algorithms";
			this.linkAlgoNone.LinkClicked += new System.Windows.Forms.LinkLabelLinkClickedEventHandler(this.OnClickAlgoNone);
			// 
			// menuFileCtxClear
			// 
			this.menuFileCtxClear.Index = 5;
			this.menuFileCtxClear.Text = "&Clear List";
			this.menuFileCtxClear.Click += new System.EventHandler(this.OnMenuFileCtxClear);
			// 
			// tabPages
			// 
			this.tabPages.Controls.Add(this.tabPageHashFiles);
			this.tabPages.Controls.Add(this.tabPageHashText);
			this.tabPages.Controls.Add(this.tabPageAlgorithms);
			this.tabPages.Controls.Add(this.tabPageOptions);
			this.tabPages.Dock = System.Windows.Forms.DockStyle.Fill;
			this.tabPages.ImageList = this.imageListMain;
			this.tabPages.Location = new System.Drawing.Point(0, 0);
			this.tabPages.Name = "tabPages";
			this.tabPages.SelectedIndex = 0;
			this.tabPages.Size = new System.Drawing.Size(552, 410);
			this.tabPages.TabIndex = 2;
			// 
			// richFileResults
			// 
			this.richFileResults.AutoWordSelection = true;
			this.richFileResults.BackColor = System.Drawing.SystemColors.Control;
			this.richFileResults.ContextMenu = this.ctxTextView;
			this.richFileResults.Dock = System.Windows.Forms.DockStyle.Fill;
			this.richFileResults.Font = new System.Drawing.Font("Courier New", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.richFileResults.Location = new System.Drawing.Point(0, 208);
			this.richFileResults.Name = "richFileResults";
			this.richFileResults.ReadOnly = true;
			this.richFileResults.Size = new System.Drawing.Size(544, 175);
			this.richFileResults.TabIndex = 1;
			this.richFileResults.Text = "Add files to the list above and click \'Compute Hashes\'.";
			// 
			// ctxFileList
			// 
			this.ctxFileList.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
						this.menuFileCtxAddFile,
						this.menuFileCtxAddDir,
						this.menuFileCtxSep1,
						this.menuFileCtxRemove,
						this.menuFileCtxSep2,
						this.menuFileCtxClear});
			// 
			// menuInfo
			// 
			this.menuInfo.Index = 1;
			this.menuInfo.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
						this.menuInfoAbout});
			this.menuInfo.Text = "&?";
			// 
			// folderBrowserToAdd
			// 
			this.folderBrowserToAdd.Description = "Select a directory which contains files to hash";
			this.folderBrowserToAdd.ShowNewFolderButton = false;
			// 
			// richTextResults
			// 
			this.richTextResults.AutoWordSelection = true;
			this.richTextResults.BackColor = System.Drawing.SystemColors.Control;
			this.richTextResults.ContextMenu = this.ctxTextView;
			this.richTextResults.Dock = System.Windows.Forms.DockStyle.Fill;
			this.richTextResults.Font = new System.Drawing.Font("Courier New", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.richTextResults.Location = new System.Drawing.Point(0, 208);
			this.richTextResults.Name = "richTextResults";
			this.richTextResults.ReadOnly = true;
			this.richTextResults.Size = new System.Drawing.Size(544, 175);
			this.richTextResults.TabIndex = 1;
			this.richTextResults.Text = "";
			// 
			// tabPageHashText
			// 
			this.tabPageHashText.Controls.Add(this.richTextResults);
			this.tabPageHashText.Controls.Add(this.panelHashTextTop);
			this.tabPageHashText.ImageIndex = 1;
			this.tabPageHashText.Location = new System.Drawing.Point(4, 23);
			this.tabPageHashText.Name = "tabPageHashText";
			this.tabPageHashText.Size = new System.Drawing.Size(544, 383);
			this.tabPageHashText.TabIndex = 1;
			this.tabPageHashText.Text = "Hash Text";
			this.tabPageHashText.ToolTipText = "Compute hashsum of a text.";
			// 
			// richTextToHash
			// 
			this.richTextToHash.AutoWordSelection = true;
			this.richTextToHash.ContextMenu = this.ctxTextView;
			this.richTextToHash.Dock = System.Windows.Forms.DockStyle.Fill;
			this.richTextToHash.Font = new System.Drawing.Font("Courier New", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.richTextToHash.Location = new System.Drawing.Point(0, 28);
			this.richTextToHash.Name = "richTextToHash";
			this.richTextToHash.Size = new System.Drawing.Size(544, 148);
			this.richTextToHash.TabIndex = 2;
			this.richTextToHash.Text = "";
			// 
			// panelHashFilesTopInner
			// 
			this.panelHashFilesTopInner.Controls.Add(this.stcHashFilesIntro);
			this.panelHashFilesTopInner.Dock = System.Windows.Forms.DockStyle.Top;
			this.panelHashFilesTopInner.Location = new System.Drawing.Point(0, 0);
			this.panelHashFilesTopInner.Name = "panelHashFilesTopInner";
			this.panelHashFilesTopInner.Size = new System.Drawing.Size(544, 28);
			this.panelHashFilesTopInner.TabIndex = 2;
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(552, 429);
			this.Controls.Add(this.tabPages);
			this.Controls.Add(this.statusMain);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Menu = this.menuMain;
			this.Name = "MainForm";
			this.Text = "VisualHash";
			this.Resize += new System.EventHandler(this.OnResize);
			this.Closed += new System.EventHandler(this.OnFormClosed);
			this.panelHashTextTopControls.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.numBaseIndent)).EndInit();
			this.panelHashFilesTop.ResumeLayout(false);
			this.tabPageAlgorithms.ResumeLayout(false);
			this.groupReportOptions.ResumeLayout(false);
			this.panelAlgoBottom.ResumeLayout(false);
			this.panelHashTextTop.ResumeLayout(false);
			this.tabPageHashFiles.ResumeLayout(false);
			this.panelHashFilesLeftButton.ResumeLayout(false);
			this.panelAlgoTop.ResumeLayout(false);
			this.panelHashFilesTopControls.ResumeLayout(false);
			this.tabPageOptions.ResumeLayout(false);
			this.panelHashTextTopIntro.ResumeLayout(false);
			this.tabPages.ResumeLayout(false);
			this.tabPageHashText.ResumeLayout(false);
			this.panelHashFilesTopInner.ResumeLayout(false);
			this.ResumeLayout(false);
		}
		#endregion

		void OnClickHashFilesCopy(object sender, System.EventArgs e)
		{
			int nSelStart = richFileResults.SelectionStart;
			int nSelLength = richFileResults.SelectionLength;
			richFileResults.SelectAll();
			richFileResults.Copy();
			richFileResults.Select(nSelStart, nSelLength); // Restore selection
		}

		void OnClickHashFilesClear(object sender, System.EventArgs e)
		{
			richFileResults.Clear();
		}

		void OnClickHashTextCopy(object sender, System.EventArgs e)
		{
			int nSelStart = richTextResults.SelectionStart;
			int nSelLength = richTextResults.SelectionLength;
			richTextResults.SelectAll();
			richTextResults.Copy();
			richTextResults.Select(nSelStart, nSelLength); // Restore selection
		}
		
		void OnClickHashTextClear(object sender, System.EventArgs e)
		{
			richTextResults.Clear();
		}
		
		uint PackCurrentFlags()
		{
			uint u = 0;

			if(checkUpperCaseHex.Checked) u |= ReHashModule.HASHFMT_UPPERCASE;

			return u;
		}
		
		void SetMainStatusText(string str)
		{
			if(str.Length == 0) statusMain.Text = "Ready.";
			else statusMain.Text = str;
		}

		void OnClickHashFiles(object sender, System.EventArgs e)
		{
			StringBuilder sb = new StringBuilder();
			string strFilePrefix = "File", strCurFile;
			uint uBaseIndent = (uint)numBaseIndent.Value;
			uint uFlags = this.PackCurrentFlags();

			if(uBaseIndent > 4) strFilePrefix = strFilePrefix.PadRight((int)uBaseIndent);
			strFilePrefix += ": <";
			
			SetMainStatusText("Hashing files... Please wait...");
			richFileResults.Text = "Hashing files... Please wait...";
			Application.DoEvents();

			ulong uTotalSize = 0, uProcessedSize = 0;
			for(int c = 0; c < lvFilesToHash.Items.Count; c++)
			{
				FileInfo fp = new FileInfo(lvFilesToHash.Items[c].Text);
				uTotalSize += (ulong)fp.Length;
			}

			for(int i = 0; i < lvFilesToHash.Items.Count; i++)
			{
				strCurFile = lvFilesToHash.Items[i].Text;

				if(ReHashTools.HashFile(ref m_hm, strCurFile))
				{
					sb.Append(strFilePrefix);
					sb.Append(strCurFile);
					sb.Append(">\r\n");
					sb.Append(ReHashTools.ReportHashes(m_hm, uBaseIndent, uFlags));
				}
				else
					sb.Append("File access error: " + lvFilesToHash.Items[i].Text + "\r\n\r\n");

				FileInfo fp = new FileInfo(strCurFile);
				uProcessedSize += (ulong)fp.Length;

				if(uTotalSize > 0)
					progressFiles.Value = (int)(100.0f * ((float)uProcessedSize / (float)uTotalSize));
				Application.DoEvents();
			}

			progressFiles.Value = 0;
			SetMainStatusText(string.Empty);

			richFileResults.Text = sb.ToString();
		}

		void OnClickHashText(object sender, System.EventArgs e)
		{
			ASCIIEncoding ascii = new ASCIIEncoding();
			uint uBaseIndent = (uint)numBaseIndent.Value;
			uint uFlags = this.PackCurrentFlags();

			byte[] pbData = ascii.GetBytes(richTextToHash.Text);

			m_hm.InitHashes();
			m_hm.UpdateHashes(pbData, (ulong)pbData.Length, true);
			m_hm.FinalizeHashes();

			richTextResults.Text = ReHashTools.ReportHashes(m_hm, uBaseIndent, uFlags);
		}

		void OnClickAlgoAll(object sender, System.Windows.Forms.LinkLabelLinkClickedEventArgs e)
		{
			for(int i = 0; i < lvAlgorithms.Items.Count; i++)
				lvAlgorithms.Items[i].Checked = true;
		}

		void OnClickAlgoNone(object sender, System.Windows.Forms.LinkLabelLinkClickedEventArgs e)
		{
			for(int i = 0; i < lvAlgorithms.Items.Count; i++)
				lvAlgorithms.Items[i].Checked = false;
		}

		void ProcessResize()
		{
			lvFilesToHash.Columns[0].Width = lvFilesToHash.ClientRectangle.Width - 1;

			float fSize = (float)lvAlgorithms.ClientRectangle.Width / 7.0F;
			lvAlgorithms.Columns[0].Width = (int)(fSize * 2.0F);
			lvAlgorithms.Columns[1].Width = (int)fSize;
			lvAlgorithms.Columns[2].Width = (int)(fSize * 4.0F);
		}

		void OnResize(object sender, System.EventArgs e)
		{
			ProcessResize();
		}

		void OnItemCheckAlgorithms(object sender, System.Windows.Forms.ItemCheckEventArgs e)
		{
			if((e.Index < 0) || (e.Index >= m_hm.m_hashes.Count)) return;

			((ReHashModule)m_hm.m_hashes[e.Index]).bEnabled = (e.NewValue == CheckState.Checked) ? true : false;
		}

		void OnFilesLvDragEnter(object sender, System.Windows.Forms.DragEventArgs e)
		{
			if(e.Data.GetDataPresent(DataFormats.FileDrop, false))
				e.Effect = DragDropEffects.Copy;
		}

		void OnFilesLvDrop(object sender, System.Windows.Forms.DragEventArgs e)
		{
			string[] aFiles = (string[])e.Data.GetData(DataFormats.FileDrop);

			foreach(string strFile in aFiles)
				lvFilesToHash.Items.Add(strFile);
		}

		void OnMenuFileExit(object sender, System.EventArgs e)
		{
			this.Close();
		}

		void OnMenuInfoAbout(object sender, System.EventArgs e)
		{
			AboutDlg dlg = new AboutDlg();
			dlg.Show();
		}

		void OnMenuFileCtxAddFile(object sender, System.EventArgs e)
		{
			DialogResult dr = openFileToAdd.ShowDialog();
			
			if(dr == DialogResult.OK)
				lvFilesToHash.Items.Add(openFileToAdd.FileName);
		}

		void OnMenuFileCtxAddDir(object sender, System.EventArgs e)
		{
			DialogResult dr = folderBrowserToAdd.ShowDialog();

			if(dr == DialogResult.OK)
			{
				string strDir = folderBrowserToAdd.SelectedPath;

				DirectoryInfo di = new DirectoryInfo(strDir);
				FileInfo[] aFiles = di.GetFiles();

				foreach(FileInfo fi in aFiles)
				{
					if((fi.Attributes & FileAttributes.Directory) != 0) continue;
					if((fi.Attributes & FileAttributes.Device) != 0) continue;

					lvFilesToHash.Items.Add(fi.FullName);
				}
			}
		}

		void OnMenuFileCtxRemove(object sender, System.EventArgs e)
		{
			ListView.SelectedListViewItemCollection lvcSel = lvFilesToHash.SelectedItems;
			int iRemoveCount = lvcSel.Count, iRemoveIndex;

			for(int i = 0; i < iRemoveCount; i++)
			{
				iRemoveIndex = lvcSel[iRemoveCount - i - 1].Index;
				lvFilesToHash.Items[iRemoveIndex].Remove();
			}
		}

		void OnMenuFileCtxClear(object sender, System.EventArgs e)
		{
			lvFilesToHash.Items.Clear();
		}

		void OnMenuTextCtxCopy(object sender, System.EventArgs e)
		{
			if(CmpCtrlEx(sender, richTextResults)) richTextResults.Copy();
			else if(CmpCtrlEx(sender, richFileResults)) richFileResults.Copy();
			else if(CmpCtrlEx(sender, richTextToHash)) richTextToHash.Copy();
			else { Debug.Assert(false); }
		}
		
		void OnMenuTextCtxPaste(object sender, System.EventArgs e)
		{
			if(CmpCtrlEx(sender, richTextResults)) richTextResults.Paste();
			else if(CmpCtrlEx(sender, richFileResults)) richFileResults.Paste();
			else if(CmpCtrlEx(sender, richTextToHash)) richTextToHash.Paste();
			else { Debug.Assert(false); }
		}

		void OnMenuTextCtxCopyAll(object sender, System.EventArgs e)
		{
			RichTextBox rtb;

			if(CmpCtrlEx(sender, richTextResults)) rtb = richTextResults;
			else if(CmpCtrlEx(sender, richFileResults)) rtb = richFileResults;
			else if(CmpCtrlEx(sender, richTextToHash)) rtb = richTextToHash;
			else { Debug.Assert(false); return; }

			int iSelStart = rtb.SelectionStart;
			int iSelLen = rtb.SelectionLength;

			rtb.SelectAll();
			rtb.Copy();
			rtb.Select(iSelStart, iSelLen); // Restore user selection
		}

		void OnMenuTextCtxClear(object sender, System.EventArgs e)
		{
			if(CmpCtrlEx(sender, richTextResults)) richTextResults.Clear();
			else if(CmpCtrlEx(sender, richFileResults)) richFileResults.Clear();
			else if(CmpCtrlEx(sender, richTextToHash)) richTextToHash.Clear();
			else { Debug.Assert(false); }
		}

		void OnFormClosed(object sender, System.EventArgs e)
		{
			ReHashModule rhm;
			string strField, strValue;

			m_xmlConfig.SetValue("Results.UpperCaseHex", checkUpperCaseHex.Checked ? "True" : "False");
			m_xmlConfig.SetValue("Results.BaseIndent", numBaseIndent.Value.ToString());

			for(int i = 0; i < m_hm.m_hashes.Count; i++)
			{
				rhm = (ReHashModule)m_hm.m_hashes[i];

				strField = "Enable-" + rhm.strShortName;
				strValue = rhm.bEnabled ? "True" : "False";

				m_xmlConfig.SetValue(strField, strValue);
			}

			m_xmlConfig.SaveToFile(m_strCfgFile);
		}

		private static bool CmpCtrlEx(object oSender, Control cWanted)
		{
			if(cWanted == null) { Debug.Assert(false); return false; }

			MenuItem mi = (oSender as MenuItem);
			if(mi != null)
			{
				ContextMenu cm = (mi.Parent as ContextMenu);
				if(cm != null)
				{
					if(cm.SourceControl == cWanted) return true;
				}
			}

			Control cSender = (oSender as Control);
			if(cSender == null) return false;

			if(cSender == cWanted) return true;
			if(cSender.Parent == cWanted) return true;
			if((cSender.Parent != null) && (cSender.Parent.Parent == cWanted))
				return true;

			return false;
		}
	}
}
