/*
  VisualHash - A Visual Hash Calculator
  Copyright (C) 2005 Dominik Reichl <oss@dominik-reichl.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

using System;

namespace VisualHash.ReHashLib
{
	public class FNV64Hash : ReHashModule
	{
		private ulong m_fnv64 = 0;

		public FNV64Hash()
		{
		}
		
		public override bool InitModule()
		{
			strName = "FNV-64";
			strShortName = "FNV-64";
			strRef = "";
			uHashSize = 64;
			uInternalBufferSize = 64;

			pbFinalHash = new byte[8];
			return true;
		}

		public override void ReleaseModule()
		{
			SecureZeroArray(pbFinalHash);
			pbFinalHash = null;
		}

		public override void InitNewHash()
		{
			m_fnv64 = 0x84222325CBF29CE4UL;
		}

		public override void UpdateHash(byte[] pbData, ulong uLength, bool bIsLastBlock)
		{
			for(ulong i = 0; i < uLength; i++)
			{
				m_fnv64 *= 0x00000100000001B3UL;
				m_fnv64 ^= pbData[i];
			}
		}

		public override void FinalizeHash()
		{
			Store64H(pbFinalHash, m_fnv64);
		}

		public override uint Test()
		{
			byte[] pb;
			byte[] r1 = { 0xF7, 0x82, 0xD9, 0x88, 0x1A, 0x2A, 0x3C, 0xCC };

			pb = StringToAsciiEx("abcabcabcabcabcabcabcabcabcabcabcabcabcabcabc");
			if(!HashAndCompare(pb, r1)) return 1;

			return 0;
		}
	}
}
