/*
  VisualHash - A Visual Hash Calculator
  Copyright (C) 2005 Dominik Reichl <oss@dominik-reichl.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

using System;
using System.Collections;

namespace VisualHash.ReHashLib
{
	public class HashManager
	{
		public ArrayList m_hashes = new ArrayList();

		public HashManager()
		{
		}

		public bool InitModules()
		{
			ReleaseModules();

			m_hashes.Add(new Adler32Hash());
			m_hashes.Add(new Cksum32Hash());
			m_hashes.Add(new CRC16Hash());
			m_hashes.Add(new CRC16CCITTHash());
			m_hashes.Add(new CRC16XModemHash());
			m_hashes.Add(new CRC32Hash());
			m_hashes.Add(new CRC32MPEG2Hash());
			m_hashes.Add(new CRC64Hash());
			m_hashes.Add(new ELF32Hash());
			m_hashes.Add(new FCS16Hash());
			m_hashes.Add(new FNV32Hash());
			m_hashes.Add(new FNV64Hash());
			m_hashes.Add(new GOSTHash());
			m_hashes.Add(new MD2Hash());
			m_hashes.Add(new MD4Hash());
			m_hashes.Add(new MD5Hash());
			m_hashes.Add(new PJW32Hash());
			m_hashes.Add(new SDBM32Hash());
			m_hashes.Add(new SHA1Hash());
			m_hashes.Add(new SHA256Hash());
			m_hashes.Add(new SHA384Hash());
			m_hashes.Add(new SHA512Hash());
			m_hashes.Add(new Size64Hash());
			m_hashes.Add(new Sum32Hash());
			m_hashes.Add(new XUM32Hash());

			ReHashModule rhm;
			bool bResult = true;

			for(int i = 0; i < m_hashes.Count; i++)
			{
				rhm = (ReHashModule)m_hashes[i];
				bResult &= rhm.InitModule();

				((ReHashModule)m_hashes[i]).bEnabled = true;
			}

			return bResult;
		}

		public void ReleaseModules()
		{
			ReHashModule rhm;

			for(int i = 0; i < m_hashes.Count; i++)
			{
				rhm = (ReHashModule)m_hashes[i];
				rhm.ReleaseModule();

				m_hashes[i] = null;
			}

			m_hashes.Clear();
		}
		
		public void InitHashes()
		{
			ReHashModule rhm;

			for(int i = 0; i < m_hashes.Count; i++)
			{
				rhm = (ReHashModule)m_hashes[i];
				if(rhm.bEnabled) rhm.InitNewHash();
			}
		}

		public void UpdateHashes(byte[] pbBuffer, ulong uDataSize, bool bIsLastBlock)
		{
			ReHashModule rhm;

			for(int i = 0; i < m_hashes.Count; i++)
			{
				rhm = (ReHashModule)m_hashes[i];
				if(rhm.bEnabled) rhm.UpdateHash(pbBuffer, uDataSize, bIsLastBlock);
			}
		}

		public void FinalizeHashes()
		{
			ReHashModule rhm;

			for(int i = 0; i < m_hashes.Count; i++)
			{
				rhm = (ReHashModule)m_hashes[i];
				if(rhm.bEnabled) rhm.FinalizeHash();
			}
		}

		public ReHashModule GetHashModule(int nModuleIndex)
		{
			if(nModuleIndex >= m_hashes.Count) return null;
			return (ReHashModule)m_hashes[nModuleIndex];
		}
	}
}
