/*
  VisualHash - A Visual Hash Calculator
  Copyright (C) 2005 Dominik Reichl <oss@dominik-reichl.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

using System;

namespace VisualHash.ReHashLib
{
	public class SDBM32Hash : ReHashModule
	{
		private uint m_h = 0;

		public SDBM32Hash()
		{
		}

		public override bool InitModule()
		{
			strName = "SDBM-32";
			strShortName = "SDBM-32";
			strRef = "";
			uHashSize = 32;
			uInternalBufferSize = 32;

			pbFinalHash = new byte[4];
			return true;
		}

		public override void ReleaseModule()
		{
			SecureZeroArray(pbFinalHash);
			pbFinalHash = null;
		}

		public override void InitNewHash()
		{
			m_h = 0;
		}

		public override void UpdateHash(byte[] pbData, ulong uLength, bool bIsLastBlock)
		{
			for(uint i = 0; i < uLength; i++)
			{
				m_h = (m_h << 6) + (m_h << 16) - m_h + pbData[i];
			}
		}

		public override void FinalizeHash()
		{
			Store32H(pbFinalHash, m_h);
		}

		public override uint Test()
		{
			return 0;
		}
	}
}
