/*
  VisualHash - A Visual Hash Calculator
  Copyright (C) 2005 Dominik Reichl <oss@dominik-reichl.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

using System;
using System.Security.Cryptography;

namespace VisualHash.ReHashLib
{
	public class SHA1Hash : ReHashModule
	{
		protected SHA1Managed m_sha1 = new SHA1Managed();

		public SHA1Hash()
		{
		}

		public override bool InitModule()
		{
			strName = "SHA-1";
			strShortName = "SHA-1";
			strRef = "FIPS PUB 180-1";
			uHashSize = 160;
			uInternalBufferSize = 512;

			// pbFinalHash = new byte[20];
			pbFinalHash = null;

			return true;
		}

		public override void ReleaseModule()
		{
			m_sha1.Clear();

			SecureZeroArray(pbFinalHash);
			pbFinalHash = null;
		}

		public override void InitNewHash()
		{
			m_sha1.Initialize();
		}

		public override void UpdateHash(byte[] pbData, ulong uLength, bool bIsLastBlock)
		{
			if(bIsLastBlock) m_sha1.TransformFinalBlock(pbData, 0, (int)uLength);
			else m_sha1.TransformBlock(pbData, 0, (int)uLength, pbData, 0);
		}

		public override void FinalizeHash()
		{
			pbFinalHash = m_sha1.Hash;
		}

		public override uint Test()
		{
			byte[] pb;
			byte[] r1 = { 0xB4, 0xC0, 0xD7, 0x18, 0xB8, 0x00, 0xE2, 0x14,
				0x80, 0x4C, 0x19, 0xB1, 0x2A, 0x63, 0x20, 0x0E, 0xF7, 0xF3, 0xFC, 0xE7 };
			byte[] r2 = { 0x84, 0x98, 0x3E, 0x44, 0x1C, 0x3B, 0xD2, 0x6E,
				0xBA, 0xAE, 0x4A, 0xA1, 0xF9, 0x51, 0x29, 0xE5, 0xE5, 0x46, 0x70, 0xF1 };
			byte[] r3 = { 0xA9, 0x99, 0x3E, 0x36, 0x47, 0x06, 0x81, 0x6A,
				0xBA, 0x3E, 0x25, 0x71, 0x78, 0x50, 0xC2, 0x6C, 0x9C, 0xD0, 0xD8, 0x9D };

			pb = StringToAsciiEx("abcabcabcabcabcabcabcabcabcabcabcabcabcabcabc");
			if(!HashAndCompare(pb, r1)) return 1;
			
			pb = StringToAsciiEx("abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq");
			if(!HashAndCompare(pb, r2)) return 2;
			
			pb = StringToAsciiEx("abc");
			if(!HashAndCompare(pb, r3)) return 3;

			return 0;
		}
	}
}
