/*
  VisualHash - A Visual Hash Calculator
  Copyright (C) 2005 Dominik Reichl <oss@dominik-reichl.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

using System;
using System.Security.Cryptography;

namespace VisualHash.ReHashLib
{
	public class SHA256Hash : ReHashModule
	{
		protected SHA256Managed m_sha256 = new SHA256Managed();

		public SHA256Hash()
		{
		}

		public override bool InitModule()
		{
			strName = "SHA-256";
			strShortName = "SHA-256";
			strRef = "FIPS PUB 180-2";
			uHashSize = 256;
			uInternalBufferSize = 512;

			// pbFinalHash = new byte[32];
			pbFinalHash = null;

			return true;
		}

		public override void ReleaseModule()
		{
			m_sha256.Clear();

			SecureZeroArray(pbFinalHash);
			pbFinalHash = null;
		}

		public override void InitNewHash()
		{
			m_sha256.Initialize();
		}

		public override void UpdateHash(byte[] pbData, ulong uLength, bool bIsLastBlock)
		{
			if(bIsLastBlock) m_sha256.TransformFinalBlock(pbData, 0, (int)uLength);
			else m_sha256.TransformBlock(pbData, 0, (int)uLength, pbData, 0);
		}

		public override void FinalizeHash()
		{
			pbFinalHash = m_sha256.Hash;
		}

		public override uint Test()
		{
			byte[] pb;
			byte[] r1 = { 0x93, 0x48, 0x38, 0x3C, 0xEC, 0x52, 0xBE, 0xAB,
				0x2B, 0xF0, 0x80, 0x4A, 0xD7, 0x9E, 0x8E, 0xE2,
				0xF0, 0x96, 0x70, 0xD9, 0x99, 0xB2, 0x11, 0x98,
				0xB4, 0xE3, 0xC2, 0x0A, 0x6A, 0x0C, 0x72, 0x97 };

			pb = StringToAsciiEx("abcabcabcabcabcabcabcabcabcabcabcabcabcabcabc");
			if(!HashAndCompare(pb, r1)) return 1;

			return 0;
		}
	}
}
