/*
  VisualHash - A Visual Hash Calculator
  Copyright (C) 2005 Dominik Reichl <oss@dominik-reichl.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

using System;
using System.Security.Cryptography;

namespace VisualHash.ReHashLib
{
	public class SHA384Hash : ReHashModule
	{
		protected SHA384Managed m_sha384 = new SHA384Managed();

		public SHA384Hash()
		{
		}

		public override bool InitModule()
		{
			strName = "SHA-384";
			strShortName = "SHA-384";
			strRef = "FIPS PUB 180-2";
			uHashSize = 384;
			uInternalBufferSize = 512;

			// pbFinalHash = new byte[48];
			pbFinalHash = null;

			return true;
		}

		public override void ReleaseModule()
		{
			m_sha384.Clear();

			SecureZeroArray(pbFinalHash);
			pbFinalHash = null;
		}

		public override void InitNewHash()
		{
			m_sha384.Initialize();
		}

		public override void UpdateHash(byte[] pbData, ulong uLength, bool bIsLastBlock)
		{
			if(bIsLastBlock) m_sha384.TransformFinalBlock(pbData, 0, (int)uLength);
			else m_sha384.TransformBlock(pbData, 0, (int)uLength, pbData, 0);
		}

		public override void FinalizeHash()
		{
			pbFinalHash = m_sha384.Hash;
		}

		public override uint Test()
		{
			byte[] pb;
			byte[] r1 = { 0xB1, 0x09, 0xF2, 0x44, 0xF7, 0x56, 0x90, 0x71,
				0xF6, 0x2A, 0x83, 0x27, 0xCF, 0xF4, 0xD6, 0xDB,
				0x46, 0xD3, 0xFD, 0xCB, 0xE9, 0x9C, 0x87, 0xB6,
				0x2D, 0x58, 0x3C, 0x24, 0x09, 0x51, 0xF0, 0x7F,
				0x45, 0xBE, 0x47, 0x7E, 0xBD, 0x2C, 0x47, 0x07,
				0x47, 0xB8, 0x15, 0xBE, 0xBE, 0xF4, 0x15, 0x8B };

			pb = StringToAsciiEx("abcabcabcabcabcabcabcabcabcabcabcabcabcabcabc");
			if(!HashAndCompare(pb, r1)) return 1;

			return 0;
		}
	}
}
